(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "dictionary_sig.sml";*)
(*require "order_sig.sml";*)
(*require "stringorder.sml";*)
(*require "tree.sml";*)
(*require "searchtree.sml";*)
(*require "gntestutil_sig.sml";*)
(*require "gntestutil.sml";*)
(*require "$.basis.__list";*)

(* General functor for dictionaries. It requires
   a structure with an ordered type as key. *)
functor FDictionary (Key : ORDER) :>
          DICTIONARY where type key = Key.t =
  struct
    structure Stree = FSearchTree (Key)

    exception NotImplemented
   
    type key = Key.t
    type 'a t = 'a Stree.t

    val empty = Tree.empty

    fun lookup tree key =
          Stree.lookup tree key

    fun insert tree key item =
          Stree.insert tree key item

    fun update tree key f =
          Stree.update tree key f

    fun delete tree key =
          Stree.delete tree key

    val size = Tree.size

    (* Now all the really powerful functions :) *)
    (* Notice that the dicionary module is not required to
       keep any particular ordering of the items.
       That's why there's only one fold function.
       Also notice that the fold function is NOT a tree fold
       but a linear fold similar to that of a list.
       Also notice that mapToList may return the elements
       in any order, but you could use mapToList for
       generating a list of (key * sortinfo), then sort
       that list and retrieve the dictionary data
       by using the keys and the lookup function.
       That's how mapSort is implemented... *)
    fun map f dict =
          Stree.map f dict

    fun filter f dict =
          Stree.filter f dict

    fun fold f init dict =
          Stree.fold f init dict

    fun mapToList f dict =
          fold (fn (a, b) => (f a)::b) nil dict

    fun sort compare itemList =
          if true then
            raise NotImplemented
          else
            itemList (* FIXME: Make a general sort routine somewhere *)

    (* FIXME: Maybe a gnOrder.compare would be ideal to use here... *)
    fun mapSort f compare dict =
          List.map f (sort compare (mapToList (fn a => a) dict))
  end

structure StringDict = FDictionary(StringOrder)

(*structure TestDictionary :> GN_TESTER =
  struct
    structure dict = FDictionary(StringOrder);

    fun sizeEmpty () =
          let
            val a = dict.empty
            val b = dict.insert a "abc" 5
            val c = dict.insert b "def" 10
            val d = dict.insert c "ghi" 15
            val e = dict.insert d "b" 7
          in
            (dict.size a = 0) andalso (dict.size b = 1) andalso
            (dict.size d = 3) andalso (dict.size e = 4)
          end

    fun lookupInsert () =
          let
            val a = dict.empty 
            val b = dict.insert a "abc" 5
            val c = dict.insert b "def" 10
            val d = dict.insert c "ghi" 15
            val e = dict.insert d "b" 7
            val tb =
                  case dict.lookup b "abc" of
                    NONE => ~1
                  | SOME(x) => x
            val tc1 =
                  case dict.lookup c "abc" of
                    NONE => ~1
                  | SOME(x) => x
            val tc2 =
                  case dict.lookup c "def" of
                    NONE => ~1
                  | SOME(x) => x
          in
            (dict.lookup a "" = NONE) andalso
            (dict.lookup a "nothing" = NONE) andalso
            (dict.lookup b "nothing" = NONE) andalso
            (dict.lookup c "nothing" = NONE) andalso
            (dict.lookup d "nothing" = NONE) andalso
            (dict.lookup e "nothing" = NONE) andalso
            (dict.lookup e "" = NONE) andalso
            (tb = 5) andalso (tc1 = 5) andalso (tc2 = 10)
          end

    fun test () =
          gnTU.doTest [("sizeEmpty", sizeEmpty),
                       ("lookupInsert", lookupInsert)]
  end*)
