(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "dgnodes";*)

infix 5 ~+;
infix 5 ~-;
infix 5 ~<+;
infix 5 ~<-;

infix 4 +:;
infix 4 +#;
(*infix 4 +@;*)

infix 4 -:;
infix 4 -#;
(*infix 4 -@;*)

functor FDGShorthand() =
  struct
    (* Modifiers - begins with a ;   *)
    

    fun (thisNode ~+ toNode) = 
          (DGNodes.connect thisNode toNode; thisNode)

    fun (thisNode ~- toNode) = 
          (DGNodes.disconnect thisNode toNode; thisNode)

    fun (thisNode ~<+ notify) =
          (DGNodes.addConnectNotify thisNode notify; thisNode)

    fun (thisNode ~<- (removableNode, notify)) =
          (DGNodes.addDisconnectNotify thisNode removableNode notify; thisNode)


    (* FIXME: Perhaps the functions in DGNodes could be made more modular/atomic
              to allow for more flexible infix operators? It might return
              functional dictionaries or somesuch... *)

    (* Connect accessors - begins with a +   *)
    

    fun (thisNode +: theType) =
          DGNodes.singleConnectedToType thisNode theType

    fun (thisNode +# theType) =
          DGNodes.connectedToType thisNode theType

    (* FIXME: This should be a unary operator with precedence *)
    fun +@ thisNode =
          DGNodes.connected thisNode



    (* Disconnect accessors - begins with a -   *)
    

    fun (thisNode -: theType) =
          DGNodes.singleReverseConnectedFromType thisNode theType

    fun (thisNode -# theType) =
          DGNodes.reverseConnectedFromType thisNode theType

    (* FIXME: This should be a unary operator with precedence *)
    fun -@ thisNode =
          DGNodes.reverseConnected thisNode
  end

structure DGShorthand = FDGShorthand()
