(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "dgnotifygeneral";*)
(*require "dgprotocols";*)
(*require "dgnodes";*)
(*require "$.typeneutral.classes";*)
(*require "protocolutils";*)
(*require "$.basis.__list";*)
(*require "$.basis.__real";*)

structure PNull =
  struct
    type t = unit
    type default = unit

    structure P =
      FClass(type newType = t
             val className = "NULL")

    fun createStd () = ()
  end

(*structure PStringRO =
  struct
    type t = (unit -> string) PMethod.t

    structure P =
      FClass(type newType = t
             val className = "String (readonly)")

    fun value prot = (PMethod.execute prot) ()
  end*)

functor FT(type tType type 'a method) =
      struct
        type t = (unit -> tType) method
      end

functor FPValueRO(type theType val className : string) =
  struct
    structure T1 = FT(type tType = theType
                      type 'a method = 'a PMethod.t)
    structure T2 = FT(type tType = theType
                      type 'a method = 'a)

    type t = T1.t
    type default = theType

    structure P =
      FClass(type newType = t
             val className = className)

    fun stdImpl (init : default) =
          (fn () => init) : T2.t

    fun createStd init =
          ref (stdImpl init) : t

    fun read prot = (PMethod.execute prot) ()
  end

structure PStringRO =
  FPValueRO(type theType = string
            val className = "String (readonly)")

functor FT(type tType type 'a method) =
      struct
        type t = {read :  (unit -> tType) method,
                  write : (tType -> unit) method,
                  (* Notice: notifier methods can't be overridden *)
                  addChangedNotifier : (DGNodes.t -> tType -> unit) ->
                    DGNotifyGeneral.notifyID,
                  removeChangedNotifier : DGNotifyGeneral.notifyID -> unit,
                  doNotifyChanged : DGNodes.t -> tType -> unit}
      end

functor FPValue(type theType
                val className : string
                val isEqual : (theType * theType) -> bool) =
  struct
    (* Most of this is a standard ritual for implementing
       protocols. It's quite ugly :(
       I'd say that protocol implementaions like these are quite
       similar to a virtual method table of an object in an OO-language.
       But with this, methods can be overrided at any time at runtime... *)
    structure T1 = FT(type tType = theType
                      type 'a method = 'a PMethod.t)
    structure T2 = FT(type tType = theType
                      type 'a method = 'a)

    type t = T1.t
    type default = theType * DGNodes.t

    structure P =
      FClass(type newType = t
             val className = className)

    fun stdImpl ((init, defSender) : default) =
          let
            val {addNotifier, removeNotifier, doNotify} =
                   DGNotifyGeneral.stdImpl ()
          
            val a = ref init
            fun doRead () = !a
            fun doWrite newValue =
                  if isEqual (!a, newValue) then
                    ()
                  else
                    (a := newValue;
                     doNotify defSender newValue)
          in
            {read = doRead, write = doWrite,
             addChangedNotifier = addNotifier,
             removeChangedNotifier = removeNotifier,
             doNotifyChanged = doNotify} : T2.t
          end

    fun createStd init =
          let
            val {read, write, addChangedNotifier,
                 removeChangedNotifier, doNotifyChanged} = stdImpl init
          in
            {read = ref read, write = ref write,
             addChangedNotifier = addChangedNotifier,
             removeChangedNotifier = removeChangedNotifier,
             doNotifyChanged = doNotifyChanged} : t
          end

    fun replaceImpl ({read, write, addChangedNotifier,
                      removeChangedNotifier, doNotifyChanged} : t)
                    ({read = newRead, write = newWrite}) =
          (read := newRead; write := newWrite)

    fun replaceWithStd defSender (prot as {read, doNotifyChanged, ...} : t) =
          let
            val init = (!read) ()
            val a = ref init
            fun doRead () = !a
            fun doWrite newValue =
                  (* FIXME: Check if the value changed? *)
                  (a := newValue;
                   doNotifyChanged defSender newValue)
          in
            replaceImpl prot {write = doWrite, read = doRead}
          end

    (* The actual protocol for the user *)
    fun read ({read, ...} : t) =
          (PMethod.execute read) ()
    fun write ({write, ...} : t) newValue =
          (PMethod.execute write) newValue
    fun addChangedNotifier ({addChangedNotifier, ...} : t) newValue =
          addChangedNotifier newValue
    fun removeChangedNotifier ({removeChangedNotifier, ...} : t) anID =
          removeChangedNotifier anID
    fun doNotifyChanged ({doNotifyChanged, ...} : t) sender value =
          doNotifyChanged sender value
  end

structure PString =
  FPValue(type theType = string
          val className = "String"
          val isEqual = (op=))

structure PInt =
  FPValue(type theType = int
          val className = "Int"
          val isEqual = (op=))

structure PReal =
  FPValue(type theType = real
          val className = "Real"
          val isEqual = Real.==)

structure PBool =
  FPValue(type theType = bool
          val className = "Bool"
          val isEqual = (op=))

structure PDGNode =
  FPValue(type theType = (DGNodes.t option)
          val className = "DGNode option"
          val isEqual = (op=))

functor FT(type tType type 'a method) =
      struct
        type t = {add :      (tType -> unit)          method,
                  insertAt : (int -> tType -> unit)   method,
                  remove :   (unit -> unit)           method,
                  deleteAt : (int -> unit)            method,
                  writeAt :  (int -> tType -> unit)   method,
                  count :    (unit -> int)            method,
                  readAt :   (int -> tType)           method,
                  clear :    (unit -> unit)           method,
                  readList : (unit -> tType list)     method,
                  (* Notice: notifier methods can't be overridden *)
                  addChangedNotifier : (DGNodes.t -> unit -> unit) ->
                    DGNotifyGeneral.notifyID,
                  removeChangedNotifier : DGNotifyGeneral.notifyID -> unit,
                  doNotifyChanged : DGNodes.t -> unit -> unit}
      end

functor FPValues(type theType val className : string) =
  struct
    structure T1 = FT(type tType = theType
                      type 'a method = 'a PMethod.t)
    structure T2 = FT(type tType = theType
                      type 'a method = 'a)

    type t = T1.t
    type default = (theType list) * DGNodes.t

    structure P =
      FClass(type newType = t
             val className = className)

    fun stdImpl (init : theType list, defSender) =
          let
            val {doNotify, addNotifier, removeNotifier} =
                   DGNotifyGeneral.stdImpl ()
          
            (* FIXME: Add exceptions etc. *)
            val a = ref init
            fun doReadAt index =
                  List.nth (!a, index)
            fun doAdd value =
                  (a := (!a) @ [value];
                   doNotify defSender ())
            fun doCount () =
                  length (!a)
            fun doClear () =
                  (a := nil;
                   doNotify defSender ())
            fun doWriteAt index newValue =
                  let
                    fun foldF (e, (cnt, acc)) =
                          if cnt = index then
                            (cnt + 1, newValue::acc)
                          else
                            (cnt + 1, e::acc)
                    val (_, result) = foldr foldF (0, nil) (!a)
                  in
                    (a := result;
                     doNotify defSender ())
                  end
            fun doDeleteAt index =
                  let
                    fun foldF (e, (cnt, acc)) =
                          if cnt = index then
                            (cnt + 1, acc)
                          else
                            (cnt + 1, e::acc)
                    val (_, result) = foldr foldF (0, nil) (!a)
                  in
                    (a := result;
                     doNotify defSender ())
                  end
            fun doInsertAt index newValue =
                  let
                    fun ins cnt nil =
                          [newValue]
                      | ins cnt (e::rest) =
                          if cnt = index then
                            newValue::e::rest
                          else
                            e::(ins (cnt + 1) rest)
                  in
                    (a := ins 0 (!a);
                     doNotify defSender ())
                  end
            fun doRemove () =
                  (a := rev (tl (rev (!a)));
                   doNotify defSender ())
            fun doReadList () = !a
          in
            {add = doAdd,
             insertAt = doInsertAt,
             remove = doRemove,
             deleteAt = doDeleteAt,
             writeAt = doWriteAt,
             count = doCount,
             readAt = doReadAt,
             clear = doClear,
             readList = doReadList,
             doNotifyChanged = doNotify,
             addChangedNotifier = addNotifier,
             removeChangedNotifier = removeNotifier} : T2.t
          end

    fun createStd (init as (_, defSender)) =
          let
            val {add,
                 insertAt,
                 remove,
                 deleteAt,
                 writeAt,
                 count,
                 readAt,
                 clear,
                 readList,
                 addChangedNotifier,
                 removeChangedNotifier,
                 doNotifyChanged} = stdImpl init

          in
            {add = ref add,
             insertAt = ref insertAt,
             remove = ref remove,
             deleteAt = ref deleteAt,
             writeAt = ref writeAt,
             count = ref count,
             readAt = ref readAt,
             clear = ref clear,
             readList = ref readList,
             addChangedNotifier = addChangedNotifier,
             removeChangedNotifier = removeChangedNotifier,
             doNotifyChanged = doNotifyChanged}
          end

    fun replaceImpl ({add, insertAt, remove, deleteAt, writeAt,
                      count, readAt, clear, readList,
                      addChangedNotifier, removeChangedNotifier,
                      doNotifyChanged} : t)
                    ({add = newAdd, insertAt = newInsertAt,
                      remove = newRemove, deleteAt = newDeleteAt,
                      writeAt = newWriteAt, count = newCount,
                      readAt = newReadAt, clear = newClear,
                      readList = newReadList}) =
          (add := newAdd; insertAt := newInsertAt;
           remove := newRemove; deleteAt := newDeleteAt;
           writeAt := newWriteAt; count := newCount;
           readAt := newReadAt; clear := newClear;
           readList := newReadList)

    fun replaceWithStd (prot as {readList, ...} : t) defSender =
          let
            val {add, insertAt, remove, deleteAt,
                 writeAt, count, readAt, clear,
                 readList, ...} = stdImpl ((!readList) (), defSender)
          in
            replaceImpl prot
               {add = add, insertAt = insertAt, remove = remove,
                deleteAt = deleteAt, writeAt = writeAt,
                count = count, readAt = readAt, clear = clear,
                readList = readList}
          end

    (* The actual protocol for the user *)
    fun readAt   ({readAt, ...} : t) index =
          (PMethod.execute readAt) index

    fun writeAt  ({writeAt, ...} : t) index newValue =
          (PMethod.execute writeAt) index newValue

    fun remove   ({remove, ...} : t) =
          (PMethod.execute remove) ()

    fun clear    ({clear, ...} : t) =
          (PMethod.execute clear) ()

    fun add      ({add, ...} : t) newValue =
          (PMethod.execute add) newValue

    fun insertAt ({insertAt, ...} : t) index newValue =
          (PMethod.execute insertAt) index newValue

    fun deleteAt ({deleteAt, ...} : t) index =
          (PMethod.execute deleteAt) index

    fun count    ({count, ...} : t) =
          (PMethod.execute count) ()

    fun clear    ({clear, ...} : t) =
          (PMethod.execute clear) ()

    fun readList ({readList, ...} : t) =
          (PMethod.execute readList) ()

    fun addChangedNotifier ({addChangedNotifier, ...} : t) newValue =
          addChangedNotifier newValue

    fun removeChangedNotifier ({removeChangedNotifier, ...} : t) anID =
          removeChangedNotifier anID

    fun doNotifyChanged (prot as ({doNotifyChanged, ...} : t)) sender =
          doNotifyChanged sender ()
  end

structure PStrings =
  FPValues(type theType = string
           val className = "Strings")

structure PInts =
  FPValues(type theType = int
           val className = "Ints")

structure PReals =
  FPValues(type theType = real
           val className = "Reals")

structure PBools =
  FPValues(type theType = bool
           val className = "Bools")

structure DGPNull = FDGProtocols(PNull)

structure DGPStringRO = FDGProtocols(PStringRO)

structure DGPString = FDGProtocols(PString)
structure DGPInt = FDGProtocols(PInt)
structure DGPReal = FDGProtocols(PReal)
structure DGPBool = FDGProtocols(PBool)
structure DGPDGNode = FDGProtocols(PDGNode)

structure DGPStrings = FDGProtocols(PStrings)
structure DGPInts = FDGProtocols(PInts)
structure DGPReals = FDGProtocols(PReals)
structure DGPBools = FDGProtocols(PBools)
