(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(* This is an attempt to minimize the verbosity required
   for calling methods in the protocol of a node. *)

(*require "dgnodes";*)
(*require "$.typeneutralwinnt.classes";*)
(*require "dgprotocols_sig";*)

functor FDGProtocols(Protocol : PROTOCOL_BASIC) =
  struct
    type t = DGNodes.t

    fun p aNode =
          DGNodes.protocol Protocol.P.c aNode

    (*fun e aNode method =
          method (DGNodes.protocol TypedProtocol.c aNode)*)
    (* Consequences of using either DGNodesProtocols.e or .p *)
    (* e: val _ = (DGPString.e aNode PString.write) "value" *)
    (* p: val _ = PString.write (DGPString.p aNode) "value" *)
    (* I think using p is clearly most readable... And using p also
       allows me to make the typecast once and use the obtained typed
       protocol many times afterwards... *)

    (* And this is for adding a protocol-parameter-node to a
       graph node with a default initial value *)
    fun add graphNode typeName default =
          let
            val prot = Protocol.createStd default

            val node = DGNodes.create typeName (Protocol.P.create prot)
            val _ = DGNodes.connect graphNode node
          in
            prot
          end
  end
