(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FPortableScrollBar(Reg : UI_REGISTER
                           where type t = uiRegGUIClass.t) : UI_PLUGIN =
  struct
    val name = "ScrollBar"
    
    fun construct () =
          let
            val (nameNode, _) = ParamName.create name

            (* FIXME: Notifications should really be done through the connections -
                      unless it is considered illegal for the user to connect
                      stuff to size, text, create, native etc. 
                      Maybe I should just add connection constraints?
                      Or notifications for stateinfo nodes? *)

            (* Add parameter nodes to the graph *)
            val scrollRangeProt =
                  DGPInt.add nameNode "ScrollRange" (100, nameNode)
            val scrollRangeUsedProt =
                  DGPInt.add nameNode "ScrollRangeUsed" (100, nameNode)
            val scrollValueProt = (* FIXME: ScrollPos??? *)
                  DGPInt.add nameNode "ScrollValue" (0, nameNode)
            val clickNProt =
                  DGPPointerButtonNotify.add nameNode "OnClick" ()
            val doubleClickNProt =
                  DGPPointerButtonNotify.add nameNode "OnDoubleClick" ()

            fun genericConstruct {invalidateRect, getArea} =
                  let
                    fun doRedraw dp rects =
                          let
                            val r = ParamRect.getRect nameNode
                            val edgeWidth = 2
                            val {x, y, w, h} = IRect2D.inset r edgeWidth
                            
                            val range = PInt.read scrollRangeProt
                            val used = PInt.read scrollRangeUsedProt
                            val value = PInt.read scrollValueProt

                            (* FIXME: Generalize to horiz/vert by
                                      exchaning x<->y and w<->h *)
                            val barHeight =
                                  if range > used then
                                    h
                                  else
                                    (h * range) div used
                            val barPos = edgeWidth + (value * h) div range
                            
                            val relRect = {x = x, y = barPos,
                                           w = w, h = barHeight}
                          in
                            (GenericLF.drawFrameIn dp r {r = 0.3, g = 0.4, b = 0.5};
                             GenericLF.drawBackground dp relRect {r = 0.4, g = 0.4, b = 0.4};
                             GenericLF.drawFrameOut dp relRect {r = 0.4, g = 0.4, b = 0.4})
                          end

                    (* FIXME: Make this standard? *)
                    fun invalidate () =
                          let
                            val {w, h, x, y} = getArea ()
                          in
                            invalidateRect {x = 0, y = 0, w = w, h = h}
                          end

                    fun doPointerDown {button, point} =
                          ()

                    fun doPointerMove {point} =
                          ()

                    fun doPointerUp {button, point} =
                          ()

                    fun wrapInt {write, read, addChangedNotifier,
                                 removeChangedNotifier, doNotifyChanged} f =
                          PMethod.wrap1Arg write invalidate
                    
                    val _ = wrapInt scrollRangeProt invalidate
                    val _ = wrapInt scrollRangeUsedProt invalidate
                    val _ = wrapInt scrollValueProt invalidate
                  in
                    {redraw = doRedraw,
                     pointerDown = doPointerDown,
                     pointerUp = doPointerUp,
                     pointerValue = (fn _ => ()),
                     pointerMove = doPointerMove,
                     keyPress = (fn _ => ())}
                  end
          
            val _ = GenericUIControl.constructInt
                      nameNode
                      {w = 18, h = 150}
                      genericConstruct
          in
            nameNode
          end

    val _ = Reg.register name construct
  end
