(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FPortableToggleButton(Reg : UI_REGISTER
                              where type t = uiRegGUIClass.t) : UI_PLUGIN =
  struct
    val name = "ToggleButton"
    
    fun construct () =
          let
            val text = "Text"

            val (nameNode, _) = ParamName.create name

            (* FIXME: Notifications should really be done through the connections -
                      unless it is considered illegal for the user to connect
                      stuff to size, text, create, native etc. 
                      Maybe I should just add connection constraints?
                      Or notifications for stateinfo nodes? *)

            (* Add parameter nodes to the graph *)
            val textProt =
                  DGPString.add nameNode text ("", nameNode)
            val toggleValueProt =
                  DGPBool.add nameNode "ToggleValue" (false, nameNode)

            fun overrideText invalidateF =
                  let
                    fun newWriteText inh value =
                          if value = PString.read textProt then
                            inh value
                          else
                            (inh value; invalidateF ())

                    val {write = writeText, ...} = textProt
                  in
                    PMethod.override1Arg writeText newWriteText
                  end

            fun genericConstruct {invalidateRect, getArea} =
                  let
                    (* FIXME: Make this a boolean node... *)
                    val buttonPressed = ref false

                    val _ = PBool.addChangedNotifier toggleValueProt
                              (fn _ => fn _ =>
                                 invalidateRect
                                   (ParamRect.getRect nameNode))
                  
                    fun doPointerDown {button, point} =
                          if !buttonPressed then
                            ()
                          else
                            (buttonPressed := true;
                             PBool.write toggleValueProt
                               (not (PBool.read toggleValueProt)))

                    fun doPointerUp {button, point} =
                          if !buttonPressed then
                            (buttonPressed := false)
                          else ()
          
                    fun doRedraw dp rects =
                          let
                            val r = ParamRect.getRect nameNode
                            val drawFrame =
                                  if PBool.read toggleValueProt then
                                    GenericLF.drawFrameIn
                                  else
                                    GenericLF.drawFrameOut
                          in
                            (drawFrame dp r {r = 0.3, g = 0.4, b = 0.5};
                             GenericLF.drawCenteredText dp r (PString.read textProt))
                          end

                    fun invalidate () =
                          let
                            val {w, h, x, y} = getArea ()
                          in
                            invalidateRect {x = 0, y = 0, w = w, h = h}
                          end

                    val _ = overrideText invalidate
                  in
                    {redraw = doRedraw,
                     pointerDown = doPointerDown,
                     pointerUp = doPointerUp,
                     pointerValue = (fn _ => ()),
                     pointerMove = (fn _ => ()),
                     keyPress = (fn _ => ())}
                  end
          
            val _ = GenericUIControl.constructInt
                      nameNode
                      {w = 100, h = 18}
                      genericConstruct
          in
            nameNode
          end

    val _ = Reg.register name construct
  end
