(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FGenUI() =
  struct
    type point = IPoint2D.t
    type rect = IRect2D.t

    type basicIn =
           {invalidateRect : rect -> unit,
            getArea : unit -> rect}

    (* FIXME: Look into this... do something
       which supports UNICODE, WideString, follows
       all the standards etc... *)
    datatype KeyData =
      KeyString of string
    | KeyCode of int
  
    type basicOut =
           {redraw : PIntDrawable.t -> rect list -> unit,
            pointerDown : {button : int, point : point} -> unit,
            pointerValue : {button : int, point : point, value : real} -> unit,
            pointerUp : {button : int, point : point} -> unit,
            pointerMove : {point : point} -> unit,
            keyPress : {symbol : KeyData} -> unit}

    type bInternal =
           {node : DGNodes.t,
            getArea : unit -> rect,
            redraw : rect list -> unit,
            pointerDown : {button : int, point : point} -> unit,
            pointerValue : {button : int, point : point, value : real} -> unit,
            pointerUp : {button : int, point : point} -> unit,
            pointerMove : {point : point} -> unit,
            keyPress : {symbol : KeyData} -> unit}
  end

structure GenUI = FGenUI()

signature GENERIC_UI_CONTROL =
  sig
    (* FIXME: rect should be the redraw rect *)
    (* Creates the generic control using the integer drawable protocol. *)
    val constructInt : DGNodes.t -> ISize2D.t ->
                       (GenUI.basicIn -> GenUI.basicOut) ->
                       unit
  end
