(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FHierarchyCore() =
  struct
    fun prepareForChildren () =
          let
            val children = ref nil
            fun registerChild newChild =
                  children :=
                    newChild::(!children)
          in
            {children = children, registerChild = registerChild}
          end
  
    fun handleDraw drawControl children rects =
          let
            (* A simple UI-control refresher... *)
            fun doRedraw ({getArea, redraw, ...} : GenUI.bInternal) =
                  let
                    val (area as {x = oX, y = oY, w, h}) =
                           getArea()
                                      
                    fun validateRect r =
                          let
                            val (intR as {x, y, w, h}) =
                                   IRect2D.intersect area r
                          in
                            if intR = IRect2D.zeroValue then
                              NONE
                            else
                              SOME({x = x - oX, y = y - oY, w = w, h = h})
                          end
                                      
                    val validRects =
                          List.mapPartial validateRect rects

                    (* FIXME: A naive union per child
                              of all refresh rects for just a
                              little better performance
                              - but we still more! *)
                    val childUnion =
                          foldl
                            (fn (a, b) => IRect2D.union a b)
                            IRect2D.zeroValue
                            validRects
                  in
                    if childUnion = IRect2D.zeroValue then
                      ()
                    else
                      redraw [childUnion]
                  end
          in
            (drawControl rects;(* Background... *)
             List.app doRedraw children)
          end
  end

structure HierarchyCore = FHierarchyCore()
