(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgnodes";*)
(*require "$.gtwinnt.dgpcommon";*)
(*require "uiregister";*)
(*require "$.basis.__list";*)
(*require "uicore_sig";*)

(* FIXME: Add dictionary for allowed UI-tasks (and maybe attributes) in the system.
          This can be used for error-checking *)

functor uiFCore(NativeCore : UI_NATIVE_CORE) :> UI_CORE =
  struct
    fun doCreate node =
          let
            val createNode = DGNodes.singleConnectedToType node "Create"
            val protocol = DGPBool.p createNode
          in
            PBool.write protocol true
          end

    val runGUI = NativeCore.runGUI
    val runApp = NativeCore.runApp
    val createMasterNode = NativeCore.createMasterNode

    fun createGUI node =
          let
            val childrenNode =
                  DGNodes.singleConnectedToType
                    node "Children"
            
            val children =
                  DGNodes.connected childrenNode
          in
            List.app (fn n => (doCreate n; createGUI n)) children
          end
  end

signature UI_PLUGIN =
  sig
    (* Currently the widgets are just expected to register themselves, but
       widgets should ideally be loaded dynamically somehow... :) *)
    val construct : unit -> DGNodes.t
  end
