(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.collectionswinnt.dictionary";*)
(*require "$.gtwinnt.dgnodes";*)

(* For internal use - this is where the imperative data are stored *)
structure uiRegistered =
  struct
    type action = unit -> unit

    val actiond = ref (StringDict.empty : action StringDict.t)

    type constructor = unit -> DGNodes.t

    val guiClassD = ref (StringDict.empty : constructor StringDict.t)

    val dummyGUIClassD = ref (StringDict.empty : constructor StringDict.t)
  end

signature UI_REGISTER =
  sig
    type t
    val register : string -> t -> unit
    val find : string -> t option
    val registered : unit -> string list
  end

functor uiRegister(type theType val dictionary : (theType StringDict.t) ref) : UI_REGISTER =
  struct
    type t = theType
    
    fun register name element =
          dictionary := StringDict.insert (!dictionary) name element

    fun find name =
          StringDict.lookup (!dictionary) name

    fun registered () =
          StringDict.fold
            (fn ((name, _), result) => name::result)
            nil
            (!dictionary)
(*
    fun debug () =
          StringDict.fold
            (fn ((name, (class, _)), _) =>
               (print ("Classname: " ^ name ^ " attributes: ");
                Excn.checkedExec (uiImplAttrGUIClass.get class "")
                  ((nil, nil), uiAbsSyn.uiDesc (uiAbsSyn.uiTasks (nil), nil, nil))
                  (GN.T.I(0));
                print "\n"))
            () (!gcd)*)
  end

structure uiRegActions =
  uiRegister(type theType = uiRegistered.action
             val dictionary = uiRegistered.actiond)

structure uiRegGUIClass =
  uiRegister(type theType = uiRegistered.constructor
             val dictionary = uiRegistered.guiClassD)

structure DummyReg =
  uiRegister(type theType = uiRegistered.constructor
             val dictionary = uiRegistered.dummyGUIClassD)
