(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

signature UI_EXENE_CORE =
  sig
    include UI_NATIVE_CORE
  end

functor FEXeneCore() :> UI_EXENE_CORE =
  struct
    structure EXB = EXeneBase

    val XDisplayName = ""

    (* Add eXene native information to the graph *)
    fun createNativeInfo display screen graphNode =
          let
            val nativeNode = EXeneDisplay.create display screen
            val _ = DGNodes.connect graphNode nativeNode
          in
            DGPEXeneDisplay.p nativeNode
          end

    fun createMasterNode () =
          let
            val (graphNode, _) = ParamName.create "MasterNode"

            (* FIXME: Close display again after use! How is another issue... *)
            val display = (EXB.openDisplay (XDisplayName, NONE))
		  handle EXB.BadAddr s => (
		    print (s ^ "\n");
		    RunCML.shutdown OS.Process.failure)

            (* Choose the current X default screen for this MasterNode *)
            val screen = EXB.defaultScreenOf display

            (* Add eXene native information to the graph *)
            val _ = createNativeInfo display screen graphNode
          in
            graphNode
          end

    val emptyMb = Mailbox.mailbox()

    (* Waiting to receive mail in an empty mailbox
       will hopefully make the thread sleep... *)
    fun runGUI masterNode =
          (Mailbox.recv emptyMb; ())

          (*while true do ()*) (* FIXME: Sleep? *)

    (* For encapsulating CML on the SMLNJ/eXene platform *)
    fun runApp f =
          let
            val _ = XDebug.init []
            val _ = RunCML.doit(
                      fn() => (XDebug.xspawn("ML::AbstractUI application", f); ()),
                      SOME(Time.fromMilliseconds 20))
          in
            ()
          end 
  end

structure EXeneCore = FEXeneCore()
