(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FPEXeneDisplay() =
  struct
    (* NOTICE: This is not implemented with PMethod like other protocols... *)
    type t = {readDisplay : unit -> EXeneBase.display,
              readScreen : unit -> EXeneBase.screen}
    type default = EXeneBase.display * EXeneBase.screen

    structure P =
      FClass(type newType = t
             val className = "EXene display")

    fun createStd ((disp, scrn) : default) =
          {readDisplay = (fn () => disp),
           readScreen = (fn () => scrn)}

    fun readDisplay ({readDisplay, ...} : t) = readDisplay ()
    fun readScreen ({readScreen, ...} : t) = readScreen ()
  end

structure PEXeneDisplay = FPEXeneDisplay ()

structure DGPEXeneDisplay = FDGProtocols(PEXeneDisplay)

functor FEXeneDisplay() =
  struct
    fun create display screen =
          let
            val protocol = PEXeneDisplay.P.create
                             (PEXeneDisplay.createStd
                                (display, screen))
          in
            (* FIXME: Maybe this should not be the direct masternode? *)
            DGNodes.create "EXene display" protocol
          end

    fun getNativeProtocol graphNode =
          let
            val nativeNode = DGNodes.singleConnectedToType
                               graphNode "EXene display"
          in
            DGPEXeneDisplay.p nativeNode
          end

    fun getNativeDisplay graphNode =
          PEXeneDisplay.readDisplay (getNativeProtocol graphNode)

    fun getNativeScreen graphNode =
          PEXeneDisplay.readScreen (getNativeProtocol graphNode)
  end

structure EXeneDisplay = FEXeneDisplay()

(* This is an attempt at a more general native eXene node *)
functor PFEXeneNative(type nativeType          (* FIXME: Create (or find) this! *)
                      val nativeName : string) (* :> P_EXENE_NATIVE *) =
  struct
    (* NOTICE: This is not implemented with PMethod like other protocols... *)
    type t = {readNative : unit -> nativeType}
    type default = nativeType

    structure P =
      FClass(type newType = t
             val className = nativeName)

    fun createStd (init : default) =
          {readNative = fn () => init}

    fun readNative ({readNative} : t) = readNative ()
  end
