(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FEXeneDrawableNode() =
  struct
    structure G = Geometry
    structure DR = Drawing
    structure EXB = EXeneBase

    fun add graphNode eXeneDrawable eXeneDisplay eXeneScreen getArea =
          let
            val pen = ref DR.defaultPen
            val currentFont =                   (* 9x15 *)
                  ref (Font.openFont eXeneDisplay "6x10") (* FIXME: Font names... *)

            fun doSetFGColor c =
                  pen := DR.updatePen
                    ((!pen), [DR.PV_Foreground (EXeneColor.toEXene eXeneScreen c)])

            (* FIXME: Offset? *)
            fun doDrawText {x, y} theText =
                  let
                    val {x = offsetX, y = offsetY, w, h} = getArea ()
                  in
                    DR.drawString eXeneDrawable
                      (!pen) (!currentFont)
                      (G.PT{x = x + offsetX, y = y + offsetY}, theText)
                  end

            (* FIXME: Make fontHeight similar to eXene? *)
            fun doTextHeight theText =
                  let
                    val {ascent, descent} =
                          Font.fontHt (!currentFont) (* FIXME: Not textHeight *)
                  in
                    ascent + descent (* FIXME: Is this even remotely correct? *)
                  end
                  
            fun doTextWidth theText =
                  Font.textWidth (!currentFont) theText

            fun doCharPositions theText =
                  Font.charPositions (!currentFont) theText

            fun doDrawLine {x1, y1, x2, y2} =
                  let
                    val {x = offsetX, y = offsetY, ...} = getArea ()
                  in
                    Drawing.drawSeg eXeneDrawable (!pen)
                      (G.LINE(G.PT{x = x1 + offsetX, y = y1 + offsetY},
                              G.PT{x = x2 + offsetX, y = y2 + offsetY}))
                  end

            fun doDrawRect {x, y, w, h} =
                  let
                    val {x = offsetX, y = offsetY, ...} = getArea ()
                  in
                    if w < 1 then
                      if w < 0 then
                        print "Width is negative in drawRect function!"
                      else ()
                    else if h < 1 then
                      if h < 0 then
                        print "Height is negative in drawRect function!"
                      else ()
                    else
                      Drawing.drawRect eXeneDrawable (!pen)
                        (G.RECT{x = x + offsetX, y = y + offsetY,
                                wid = w - 1, ht = h - 1})
                    (* NOTICE: eXene (and X) rects are 1 pixel bigger *)
                  end

            fun doFillRect {x, y, w, h} =
                  let
                    val {x = offsetX, y = offsetY, ...} = getArea ()
                  in
                    if w < 0 then
                      print "Width is negative in drawRect function!"
                    else if h < 0 then
                      print "Height is negative in drawRect function!"
                    else
                      Drawing.fillRect eXeneDrawable (!pen)
                        (G.RECT{x = x + offsetX, y = y + offsetY,
                                wid = w, ht = h})
                  end

            val protocol = PIntDrawable.createStd ()
            val _ = PIntDrawable.replaceImpl protocol
                      {drawLine = doDrawLine,
                       drawRect = doDrawRect,
                       fillRect = doFillRect,
                       setFGColor = doSetFGColor,
                       drawText = doDrawText,
                       textHeight = doTextHeight,
                       textWidth = doTextWidth,
                       charPositions = doCharPositions}
            
            val newNode = DGNodes.create "Integer drawable"
                            (PIntDrawable.P.create protocol)
            val _ = DGNodes.connect graphNode newNode

            (* FIXME: This is just temporary... I'm setting the
                      default color here... *)
            val _ = PIntDrawable.setFGColor protocol {r = 1.0, g = 1.0, b = 1.0}
          in
            protocol
          end

    fun getNativeProtocol graphNode =
          let
            val nativeNode = DGNodes.singleConnectedToType
                               graphNode "Integer drawable"
          in
            DGPIntDrawable.p nativeNode
          end
  end

structure EXeneDrawableNode = FEXeneDrawableNode()
structure ParamDrawableNode = EXeneDrawableNode (* FIXME: ??? *)
