(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgshorthand";*)
(*require "pnotifiers";*)
(*require "$.gtwinnt.dgpcommon";*)

functor FLFNMiscNotify() =
  struct
    local
      open DGShorthand

      (* NOTICE: The functions in LFNMiscNotify will possibly
                 be moved elsewhere... *)
      
      (* FIXME: Use higher-order functions here and put it in
                a general place... *)
      fun addStringNotifier graphNode notifyName value =
            let
              val notifyNode = graphNode +: notifyName
              val anID = PStringNotify.addNotifier
                           (DGPStringNotify.p notifyNode)
                           value
            in (graphNode, anID) end

      fun removeStringNotifier graphNode notifyName anID =
            let
              val notifyNode = graphNode +: notifyName
              val _  = PStringNotify.removeNotifier
                         (DGPStringNotify.p notifyNode)
                         anID
            in () end

      fun addStringChangedNotifier graphNode notifyName value =
            let
              val notifyNode = graphNode +: notifyName
              val anID = PString.addChangedNotifier
                           (DGPString.p notifyNode)
                           value
            in (graphNode, anID) end

      fun removeStringChangedNotifier graphNode notifyName anID =
            let
              val notifyNode = graphNode +: notifyName
              val _  = PString.removeChangedNotifier
                         (DGPString.p notifyNode)
                         anID
            in () end

      fun addBoolNotifier graphNode notifyName value =
            let
              val notifyNode = graphNode +: notifyName
              val anID = PBoolNotify.addNotifier
                           (DGPBoolNotify.p notifyNode)
                           value
            in (graphNode, anID) end

      fun removeBoolNotifier graphNode notifyName anID =
            let
              val notifyNode = graphNode +: notifyName
              val _  = PBoolNotify.removeNotifier
                         (DGPBoolNotify.p notifyNode)
                         anID
            in () end

      fun addBoolChangedNotifier graphNode notifyName value =
            let
              val notifyNode = graphNode +: notifyName
              val anID = PBool.addChangedNotifier
                           (DGPBool.p notifyNode)
                           value
            in (graphNode, anID) end

      fun removeBoolChangedNotifier graphNode notifyName anID =
            let
              val notifyNode = graphNode +: notifyName
              val _  = PBool.removeChangedNotifier
                         (DGPBool.p notifyNode)
                         anID
            in () end

      fun addNotifier graphNode notifyName value =
            let
              val notifyNode = graphNode +: notifyName
              val anID = PNotify.addNotifier
                           (DGPNotify.p notifyNode)
                           value
            in (graphNode, anID) end

      fun removeNotifier graphNode notifyName anID =
            let
              val notifyNode = graphNode +: notifyName
              val _  = PNotify.removeNotifier
                         (DGPNotify.p notifyNode)
                         anID
            in () end
    in
      fun addActionNotifier graphNode value =
            addNotifier graphNode "OnAction" value

      fun removeActionNotifier graphNode anID =
            removeNotifier graphNode "OnAction" anID

      fun addTextChangedNotifier graphNode value =
            addStringChangedNotifier graphNode "Text" value

      fun removeTextChangedNotifier graphNode anID =
            removeStringChangedNotifier graphNode "Text" anID

      fun addEditableTextChangedNotifier graphNode value =
            addStringChangedNotifier graphNode "EditableText" value

      fun removeEditableTextChangedNotifier graphNode anID =
            removeStringChangedNotifier graphNode "EditableText" anID

      fun addFocusChangedNotifier graphNode value =
            addBoolChangedNotifier graphNode "HasFocus" value

      fun removeFocusChangedNotifier graphNode anID =
            removeBoolChangedNotifier graphNode "HasFocus" anID

      fun addToggleValueChangedNotifier graphNode value =
            addBoolChangedNotifier graphNode "ToggleValue" value

      fun removeToggleValueChangedNotifier graphNode anID =
            removeBoolChangedNotifier graphNode "ToggleValue" anID
    end
  end

structure LFNMiscNotify = FLFNMiscNotify()
