(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgprotocols";*)
(*require "$.typeneutralwinnt.classes";*)
(*require "$.gtwinnt.protocolutils";*)
(*require "$.geometrywinnt.simplegeom2d";*)

functor FPIntDrawableT(type 'a method) =
      struct
        (* These are some primitive GUI types *)
        type point = IPoint2D.t
        type line = ILine2D.t
        type rect = IRect2D.t
        type color = {r : real, g : real, b : real}

        type t =
          {drawLine : (line -> unit) method,
           drawRect : (rect -> unit) method,
           fillRect : (rect -> unit) method,
           setFGColor : (color -> unit) method,
           drawText : (point -> string -> unit) method,
           textHeight : (string -> int) method,
           textWidth : (string -> int) method,
           charPositions : (string -> int list) method}
      end

(* This is a drawable protocol for integer coordinates.
   The Right Thing (TM) is of course real coordinates. *)
functor FPIntDrawable() =
  struct
    structure T1 = FPIntDrawableT(type 'a method = 'a PMethod.t)
    structure T2 = FPIntDrawableT(type 'a method = 'a)

    type t = T1.t
    type default = unit

    structure P =
      FClass(type newType = t
             val className = "Integer drawable")

    fun stdImpl _ =
          let
            fun doNothing _ = ()
          in
            {drawLine = doNothing,
             drawRect = doNothing,
             fillRect = doNothing,
             setFGColor = doNothing,
             drawText = fn _ => fn _ => (),
             textHeight = fn _ => 0,
             textWidth = fn _ => 0,
             charPositions = fn _ => nil} : T2.t
          end

    fun createStd init =
          let
            val {drawLine, drawRect,
                 fillRect, setFGColor,
                 drawText, textHeight,
                 textWidth, charPositions} = stdImpl init
          in
            {drawLine = ref drawLine,
             drawRect = ref drawRect,
             fillRect = ref fillRect,
             setFGColor = ref setFGColor,
             drawText = ref drawText,
             textHeight = ref textHeight,
             textWidth = ref textWidth,
             charPositions = ref charPositions} : t
          end

    fun replaceImpl ({drawLine, drawRect,
                      fillRect, setFGColor,
                      drawText, textHeight,
                      textWidth, charPositions} : t)
                    ({drawLine = newDrawLine,
                      drawRect = newDrawRect,
                      fillRect = newFillRect,
                      setFGColor = newSetFGColor,
                      drawText = newDrawText,
                      textHeight = newTextHeight,
                      textWidth = newTextWidth,
                      charPositions = newCharPositions} : T2.t) =
          (drawLine := newDrawLine;
           drawRect := newDrawRect;
           fillRect := newFillRect;
           setFGColor := newSetFGColor;
           drawText := newDrawText;
           textHeight := newTextHeight;
           textWidth := newTextWidth;
           charPositions := newCharPositions)

    fun replaceWithStd (prot : t) =
          replaceImpl prot (stdImpl ())

    (* The actual protocol for the user. This protocol is still very incomplete... *)
    fun drawLine ({drawLine, ...} : t) theLine = (PMethod.execute drawLine) theLine
    fun drawRect ({drawRect, ...} : t) theRect = (PMethod.execute drawRect) theRect
    fun fillRect ({fillRect, ...} : t) theRect = (PMethod.execute fillRect) theRect
    fun setFGColor ({setFGColor, ...} : t) theColor = (PMethod.execute setFGColor) theColor
    fun drawText ({drawText, ...} : t) aPos theText = (PMethod.execute drawText) aPos theText
    fun textHeight ({textHeight, ...} : t) theText = (PMethod.execute textHeight) theText
    fun textWidth ({textWidth, ...} : t) theText = (PMethod.execute textWidth) theText
    fun charPositions ({charPositions, ...} : t) theText = (PMethod.execute charPositions) theText
  end

structure PIntDrawable = FPIntDrawable ()

structure DGPIntDrawable = FDGProtocols(PIntDrawable)
