(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgprotocols";*)
(*require "$.gtwinnt.dgpcommon";*)
(*require "$.gtwinnt.dgnodes";*)
(*require "$.typeneutralwinnt.classes";*)
(*require "$.gtwinnt.protocolutils";*)
(*require "$.gtwinnt.dgnotifygeneral";*)
(*require "$.basis.__list";*)
(*require "$.geometrywinnt.simplegeom2d";*)

(* FIXME: Move this to a more general place *)
functor FPNotifyT(type 'a method type notifyParam) =
      struct
        (* These are some primitive notification types *)
        (* FIXME: More general? *)
        type notifyID = unit ref
        
        type t =
          {addNotifier : ((DGNodes.t -> notifyParam -> unit) -> notifyID) method,
           doNotify : (DGNodes.t -> notifyParam -> unit) method,
           removeNotifier : (notifyID -> unit) method}
      end

(* This is a drawable protocol for integer coordinates.
   The Right Thing (TM) is of course real coordinates. *)
(* FIXME: Make a general parametrized notify functor... *)
functor FPNotify(type notifyParam val className : string) =
  struct
    structure T1 = FPNotifyT(type 'a method = 'a PMethod.t
                             type notifyParam = notifyParam)
    structure T2 = FPNotifyT(type 'a method = 'a
                             type notifyParam = notifyParam)

    type t = T1.t
    type default = unit

    structure P =
      FClass(type newType = t
             val className = className)

    val stdImpl = DGNotifyGeneral.stdImpl

    fun createStd init =
          let
            val {addNotifier, removeNotifier,
                 doNotify} = stdImpl init
          in
            {addNotifier = ref addNotifier,
             removeNotifier = ref removeNotifier,
             doNotify = ref doNotify} : t
          end

    fun replaceImpl ({addNotifier, removeNotifier,
                      doNotify} : t)
                    ({addNotifier = newAddNotifier,
                      removeNotifier = newRemoveNotifier,
                      doNotify = newDoNotify} : T2.t) =
          (addNotifier := newAddNotifier;
           removeNotifier := newRemoveNotifier;
           doNotify := newDoNotify)

    fun replaceWithStd (prot : t) =
          replaceImpl prot (stdImpl ())

    (* The actual protocol for the user. *)
    fun addNotifier ({addNotifier, ...} : t) theNotifier = (PMethod.execute addNotifier) theNotifier
    fun removeNotifier ({removeNotifier, ...} : t) theID = (PMethod.execute removeNotifier) theID
    fun doNotify ({doNotify, ...} : t) sender param = (PMethod.execute doNotify) sender param
  end

(* FIXME: Move this to a more specific place *)
structure PPointerButtonNotify =
  FPNotify (type notifyParam = {button : int,
                               point : IPoint2D.t}
            val className = "PointerButtonNotify")

structure DGPPointerButtonNotify = FDGProtocols(PPointerButtonNotify)

structure PStringNotify =
  FPNotify (type notifyParam = string
            val className = "StringNotify")

structure DGPStringNotify = FDGProtocols(PStringNotify)

structure PBoolNotify =
  FPNotify (type notifyParam = bool
            val className = "BoolNotify")

structure DGPBoolNotify = FDGProtocols(PBoolNotify)

structure PNotify =
  FPNotify (type notifyParam = unit
            val className = "Notify")

structure DGPNotify = FDGProtocols(PNotify)
