(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gtwinnt.dgprotocols";*)
(*require "$.typeneutralwinnt.classes";*)
(*require "$.gtwinnt.protocolutils";*)
(*require "$.basis.__list";*)
(*require "$.collectionswinnt.simplesets";*)

(* FIXME: Consider creating a more general protocol for fixed-size
   arrays instead of this. I will first have to examine what this
   protocol really needs to do though... *)

functor FPItemSelectionsT(type 'a method) =
      struct
        type t = {isSelectedAt : (int -> bool) method,
                  selectAt : (int -> unit) method,
                  deselectAll : (unit -> unit) method,
                  count :        (unit -> int) method}
      end

functor FPItemSelections(val className : string) =
  struct
    (* Most of this is a standard ritual for implementing
       protocols. It's quite ugly :(
       I'd say that protocol implementaions like these are quite
       similar to a virtual method table of an object in an OO-language.
       But with this, methods can be overrided at any time at runtime... *)
    structure T1 = FPItemSelectionsT(type 'a method = 'a PMethod.t)
    structure T2 = FPItemSelectionsT(type 'a method = 'a)

    type t = T1.t
    type default = int list

    structure P =
      FClass(type newType = t
             val className = className)

    fun stdImpl (init : default) =
          let
            val a = ref nil
            fun doIsSelectedAt index =
                  SimpleSets.contains index (!a)
            fun doCount () = 0
            fun doDeselectAll () =
                  a := nil
            fun doSelectAt index =
                  if SimpleSets.contains index (!a) then
                    ()
                  else
                    a := index::(!a)
          in
            {isSelectedAt = doIsSelectedAt,
             selectAt = doSelectAt,
             deselectAll = doDeselectAll,
             count = doCount} : T2.t
          end

    fun createStd init =
          let
            val {isSelectedAt, selectAt, deselectAll, count} = stdImpl init
          in
            {isSelectedAt = ref isSelectedAt,
             selectAt = ref selectAt,
             deselectAll = ref deselectAll,
             count = ref count} : t
          end

    fun replaceImpl ({isSelectedAt, selectAt,
                      deselectAll, count} : t)
                    ({isSelectedAt = newIsSelectedAt,
                      selectAt = newSelectAt,
                      deselectAll = newDeselectAll,
                      count = newCount} : T2.t) =
          (isSelectedAt := newIsSelectedAt;
           selectAt := newSelectAt;
           deselectAll := newDeselectAll;
           count := newCount)

    fun replaceWithStd (prot : t) =
          replaceImpl prot (stdImpl nil) (* FIXME: Read and transfer values... *)

    (* The actual protocol for the user *)
    fun isSelectedAt ({isSelectedAt, ...} : t) index =
          (PMethod.execute isSelectedAt) index
    fun selectAt ({selectAt, ...} : t) index =
          (PMethod.execute selectAt) index
    fun count ({count, ...} : t) =
          (PMethod.execute count) ()
    fun deselectAll ({deselectAll, ...} : t) =
          (PMethod.execute deselectAll) ()
  end

structure PItemSelections =
  FPItemSelections(val className = "ItemSelections")

structure DGPItemSelections = FDGProtocols(PItemSelections)
