(*
    This file is part of ML::AbstractUI - an abstract user interface
    API in Standard ML with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gnLibwinnt.gn";*)
(*require "$.gnlibwinnt.gnmlworks";*)
(*require "$.mlabstractuiwinnt.uiregister";*)
(*require "$.mlabstractuiwinnt.uiwindowstoportable";*)
(*require "$.gtwinnt.dgnodes";*)
(*require "$.gtwinnt.dgshorthand";*)
(*require "$.typeneutralprotocols.simpleprotocols";*)
(*require "$.mlabstractuiwinnt.winwindow";*)
(*require "uiwingroupbox";*)
(*require "$.mlabstractuiwinnt.winbutton";*)
(*require "$.mlabstractuiwinnt.winlistbox";*)
(*require "uiwintextfield";
(*require "uiwintextlines";*)
(*require "uiwintextlabel";*)
(*require "uitreeview";*)*)
(*require "$.basis.__text_io";*)
(*require "$.basis.__int";*)

(* test data *)
structure HyperDemo =
  struct
    open DGShorthand
    exception Error 

    fun createGraph () =
          let
            (* Create the nodes *)
            val master = uiCore.createMasterNode ()
            val win = Reg.createA "Window"
            val button = Reg.createA "Button"
            val button2 = Reg.createA "Button"
            val visButton = Reg.createA "Button"
            val listbox = Reg.createA "ListBox"
            val scrollbar = Reg.createA "ScrollBar"

            (* Get the Children nodes *)
            val masterCh = master +: "Children"
            val winCh = win +: "Children"

            (* Connect the nodes *)
            val _ = masterCh ~+ win
            val _ = winCh ~+ button
                          ~+ button2
                          ~+ visButton
                          ~+ listbox
                          ~+ scrollbar

            (* Initialize data *)
            (* FIXME: Implement a keepProtocolConnectionUpdated thingy... *)
            val buttonLabel = button +: "Text"
            val buttonLabelP = DGPString.p buttonLabel
            val _ = PString.write buttonLabelP "Static"

            val _ = LFNSimpleText.setText button2 "Clear listbox"
            val _ = LFNSimpleText.setText visButton "Visibility"
            val _ = LFNSimpleText.setText win "ML::AbstractUI Demo"

            val scrRange = scrollbar +: "ScrollRange"
            val _ = PInt.write (DGPInt.p scrRange) 100

            val scrRangeU = scrollbar +: "ScrollRangeUsed"
            val _ = PInt.write (DGPInt.p scrRangeU) 135

            val _ = LFNGeometry.setPosX button2 150
            val _ = LFNGeometry.setPosX visButton 250
            val _ = LFNGeometry.setPosX scrollbar 300
            val _ = LFNGeometry.setPosY scrollbar 50
            val _ = LFNGeometry.setPosY listbox 50

            val listboxLines = listbox +: "TextLines"
            val listboxLinesP = DGPStrings.p listboxLines

            val listboxVis = listbox +: "Visible"
            val listboxVisP = DGPBool.p listboxVis

            val cnt = ref 0

            fun actionF _ () =
                  (cnt := (!cnt) + 1;
                   PStrings.insertAt listboxLinesP 0 "line...";
                   PString.write buttonLabelP ("Dynamic " ^ Int.toString (!cnt)))

            fun clearAction _ () =
                  PStrings.clear listboxLinesP

            fun visibleAction _ () =
                  PBool.write listboxVisP (not (PBool.read listboxVisP))

            (* Setup actions *)
            val _ = LFNMiscNotify.addActionNotifier button actionF
            val _ = LFNMiscNotify.addActionNotifier button2 clearAction
            val _ = LFNMiscNotify.addActionNotifier visButton visibleAction
          in
            master
          end

    fun theApp () =
          let
            val graph = createGraph ()
            val _ = Excn.checkedExec uiCore.createGUI graph ()
          in
            uiCore.runGUI graph
          end

    fun doIt () =
          uiCore.runApp theApp
  end

(*fun deliver () =
      MLWorks.Deliver.deliver ("HyperDemo.exe", HyperDemo.doIt, MLWorks.Deliver.WINDOWS);*)
