(*
    This file is part of ML::AbstractUI - an abstract user interface
    API in Standard ML with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.gnLibwinnt.gn";*)
(*require "$.gnlibwinnt.gnmlworks";*)
(*require "uiregister";*)
(*require "uiwindowstoportable";*)
(*require "$.gtwinnt.dhnodes";*)
(*require "$.typeneutralprotocols.simpleprotocols";*)
(*require "winwindow";*)
(*require "uiwingroupbox";*)
(*require "winbutton";*)
(*require "winlistbox";*)
(*require "uiwintextfield";
(*require "uiwintextlines";*)
(*require "uiwintextlabel";*)
(*require "uitreeview";*)*)
(*require "$.basis.__text_io";*)
(*require "$.basis.__int";*)

(* test data *)
structure ListboxLinesDemo =
  struct
    exception Error 

    val count = ref ~1000
    fun init value =
          count := ~value

    fun createGraph () =
          let
            open DGShorthand
          
            val master = uiCore.createMasterNode ()
            val win = Reg.createA "Window"
            val listbox = Reg.createA "ListBox"
            val button = Reg.createA "Button"
            val textField = Reg.createA "TextField"

            (* Get the Children nodes *)
            val masterCh = master +: "Children"
            val winCh = win +: "Children"

            val _ = masterCh ~+ win
            val _ = winCh ~+ listbox ~+ button ~+ textField

            val _ = LFNGeometry.setSizeX win 200
            val _ = LFNGeometry.setSizeY win 200

            val total = ~(!count)
            val _ = LFNSimpleText.setText win ("ML::AbstractUI adds listboxlines")

            val listboxLines = listbox +: "TextLines"
            val listboxLinesP = DGPStrings.p listboxLines

            val _ = LFNGeometry.setPosX textField 100
            val _ = LFNGeometry.setPosY listbox 20
            val _ = LFNSimpleText.setEditableText textField "100"
            val _ = LFNSimpleText.setText button "Create!"

            fun createOne toGo =
                  let
                    val _ = PStrings.add listboxLinesP ("Line number: " ^ (Int.toString toGo))
                  in
                    if toGo > 0 then
                      createOne (toGo - 1)
                    else ()
                  end

            fun doCreate _ () =
                  case Int.fromString
                          (PString.read
                             (DGPString.p
                                (textField +: "EditableText"))) of
                    NONE => ()
                  | SOME(number) =>
                      createOne number

            val _ = LFNMiscNotify.addActionNotifier button doCreate
          in
            master
          end

    fun theApp () =
          let
            val graph = createGraph ()
            val _ = Excn.checkedExec uiCore.createGUI graph ()
          in
            uiCore.runGUI graph
          end

    fun doIt () =
          uiCore.runApp theApp
  end

(*fun deliverLinesTest () =
      MLWorks.Deliver.deliver ("ListboxLinesDemo.exe", ListboxLinesDemo.doIt, MLWorks.Deliver.WINDOWS);
*)
