(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.basis.__string";*)
(*require "$.collectionswinnt.dictionary";*)
(*require "stringres_sig";*)
(*require "$.orderedtypeswinnt.order_sig";*)

(* This is a ressource manager for strings. It minimizes storage
   requirements for strings in the system. *)
functor FStringRes() :> STRING_RES =
  struct
    type t = string ref

    fun compareInternal (value1, value2) =
          String.compare (!value1, !value2)

    structure KeyInternal : ORDER =
      struct
        type t = string ref
        val compare = compareInternal
      end

    structure D = FDictionary(KeyInternal);

    val compare = compareInternal

    val dict = ref(D.empty : t D.t) (* FIXME: Does global references work? *)

    fun create newString =
          let
            val newRef = ref(newString)
          in
            case D.lookup (!dict) newRef of
              NONE => (dict := D.insert (!dict) newRef newRef; newRef)
            | SOME(foundRef) => foundRef
          end

    (* Used only for very temporary strings - like
       when searching in a dictionary *)
    fun createTmp newString = ref newString

    fun value stringRef =
          !stringRef
  end

structure StringRes = FStringRes()
structure NamedValues = FDictionary(StringRes) (* FIXME: Should this be here? *)
  (* FIXME: NamedValuesRes? or StringResDict? *)
  (* FIXME: Should NamedValues even exist? *)
