(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation in Standard ML.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "lexerrormsg";*)
(*require "$.basis.__real";*)
(*require "$.basis.__int";*)
(*require "$.basis.__substring";*)
(*require "$.basis.__string";*)
(*require "$.basis.__string_cvt";*)

structure LexMisc =
  struct
    type linenum = int
    (* type fileRef = string ref? *)
    type pos = int
    (* FIXME: Not used... *)
    datatype fileLoc = fLoc of int * int (* * fileRef? *)

    fun fileLoc (pos, rg) = (pos, pos + rg)
    fun valueFileLoc (value, pos, rg) = ((value, pos, pos + rg))

    fun cvtErr str pos numType =
          (ErrorMsg.error pos
            ("error converting "^numType^" literal "^str))

    fun cvtString radix (str, pos) = 
          (case (Int.scan
                   radix
                   Substring.getc
                   (Substring.all str)) of
             NONE => ((cvtErr str pos "integer");0)
           | SOME (a, _) => a)
               handle Overflow =>
                 ((cvtErr str pos "integer");0)

    val hexStringToInt = cvtString StringCvt.HEX;
    val octStringToInt = cvtString StringCvt.OCT;
    val decStringToInt = cvtString StringCvt.DEC;

    fun decStringToReal (str, pos) =
          (case Real.fromString str of
             NONE    => ((cvtErr str pos "real");0.0)
           | SOME(a) => a)
               handle Overflow =>
                 ((cvtErr str pos "real");0.0)

    (* Trims a string by triml characters at the beginning
       of the string and trimr at the end of the string.  *)
    fun trimlr triml trimr str =
          String.substring
            (str, triml, (size str) - triml - trimr)

    (* Trims both ends of a string as you can see... *)
    fun trimEsc trim str =
          trimlr (trim+1) trim str


  end (* structure LexMisc *)
