(*
    This file is part of ML::AbstractUI - an abstract user interface
    API with a crossplatform implementation.

    Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen).
    
    E-mail: anoq@HardcoreProcessing.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(*require "$.typeneutralwinnt.classes";*)
(*require "uiprotocols";*)
(*require "$.gtwinnt.dgnodes";*)
(*require "$.gtwinnt.dgpcommon";*)
(*require "pnotifiers";*)

functor FParamCreate() =
  struct
    fun createCreateNode nameNode constr destr =
          let
            val (prot as {write, ...}) = PBool.createStd (false, nameNode)
            val writeF = !write
            fun newWrite value =
                  if PBool.read prot = value then
                    ()
                  else
                    case value of
                      true => (constr (); writeF true)
                    | false => (destr (); writeF false)

            (* Override default write method *)
            val _ = write := newWrite
          in
            (* Create the graph node *)
            DGNodes.create "Create" (PBool.P.create prot)
          end

    fun setup graphNode constr destr =
          DGNodes.connect graphNode
            (createCreateNode graphNode constr destr)
  end

structure ParamCreate = FParamCreate()

functor FParamDefaultInteger(val nodePreName : string) =
  struct
    fun add graphNode name default =
          let
            val prot = PInt.createStd default

            val node = DGNodes.create name (PInt.P.create prot)
            val _ = DGNodes.connect graphNode node
          in
            prot
          end

     fun addXY graphNode defX defY =
           let
             val protX = add graphNode (nodePreName ^ "X") (defX, graphNode)
             val protY = add graphNode (nodePreName ^ "Y") (defY, graphNode)
           in (protX, protY) end

     fun get graphNode name =
           let
             val valueNodes = DGNodes.connected graphNode

             fun readIt valueNode =
                   PInt.read (DGPInt.p valueNode)
             
             fun findIt (valueNode, result) =
                   if DGNodes.typeName valueNode = name then
                     readIt valueNode
                   else result
           in
             foldl findIt 0 valueNodes
           end

     fun getXY graphNode =
           (get graphNode (nodePreName ^ "X"),
            get graphNode (nodePreName ^ "Y"))
  end

structure ParamSize = FParamDefaultInteger(val nodePreName = "Size")
structure ParamPos = FParamDefaultInteger(val nodePreName = "Pos")
structure ParamInt = FParamDefaultInteger(val nodePreName = "")

functor FParamRect() =
  struct
    (* This is relative to the parent's coordinatesystem. *)
    fun addRect graphNode {x, y, w, h} =
          let
            val (protSizeX, protSizeY) =
                  ParamSize.addXY graphNode w h
            val (protPosX, protPosY) =
                  ParamPos.addXY graphNode x y

            fun getArea () =
                  {x = PInt.read protPosX, y = PInt.read protPosY,
                   w = PInt.read protSizeX, h = PInt.read protSizeY}
          in
            {protSizeX = protSizeX, protSizeY = protSizeY,
             protPosX = protPosX, protPosY = protPosY,
             getArea = getArea}
          end

    (* FIXME: This is relative to the control's own
       coordinate system...
       (currently used by the drawing routines) *)
    fun getRect graphNode =
          let
            val (x, y) = ParamPos.getXY graphNode
            val (w, h) = ParamSize.getXY graphNode
          in
            {x = 0, y = 0, w = w, h = h}
          end
  end

structure ParamRect = FParamRect()

functor FParamName() =
  struct
    fun create name =
          let
            val childrenNode = DGNodes.create "Children" (PNull.P.create ())
            (* FIXME: Sender = self? *)
            val prot = PString.createStd (name, childrenNode)
            val node = DGNodes.create name (PString.P.create prot)
            val _ = DGNodes.connect node childrenNode
          in
            (node, prot)
          end
  end

structure ParamName = FParamName()
